@@ -9,12 +9,9 @@
 /*                                                                       */
 /*************************************************************************/
 
-#ifdef USE_ANYSPLIT
-
 /**
  * anysplit.c -- code that splits words into random morphemes.
  * This is used for the language-learning/morpheme-learning project.
- * This code is conditionally compiled, because ...
  */
 
 /* General assumptions:
@@ -57,7 +54,7 @@ typedef struct split_cache /* split cached by word length */
 #endif
 typedef struct anysplit_params
 {
-	int nparts;                /* maximum number of parts to split to */
+	int nparts;                /* maximum number of suffixes to split to */
 	size_t altsmin;            /* minimum number of alternatives to generate */
 	size_t altsmax;            /* maximum number of alternatives to generate */
 	Regex_node *regpre, *regmid, *regsuf; /* issue matching combinations  */
@@ -205,6 +202,7 @@ void free_anysplit(Dictionary afdict)
 	free_regexs(as->regmid);
 	free_regexs(as->regsuf);
 	free(as);
+	afdict->anysplit = NULL;
 }
 
 /*
@@ -315,6 +313,7 @@ static Regex_node * regbuild(const char **regstring, int n, int classnum)
 		new_re->pattern = s;
 		new_re->re      = NULL;
 		new_re->next    = NULL;
+		new_re->neg = false; /* TODO (if needed): Negative regex'es. */
 		*tail = new_re;
 		tail	= &new_re->next;
 	}
@@ -322,7 +321,8 @@ static Regex_node * regbuild(const char **regstring, int n, int classnum)
 }
 
 
-/* Affix classes:
+/**
+ * Affix classes:
  * REGPARTS  Max number of word partitions. Value 0 disables anysplit.
  * REGPRE    Regex for prefix
  * REGMID    Regex for middle suffixes
@@ -335,6 +335,9 @@ static Regex_node * regbuild(const char **regstring, int n, int classnum)
 
 /**
  * Initialize the anysplit parameter and cache structure.
+ * Return true if initialization succeeded, or if dictionary does not use
+ * anysplit (its not an error to not use anysplit!).  Return false if
+ * init failed.
  */
 bool anysplit_init(Dictionary afdict)
 {
@@ -378,12 +381,14 @@ bool anysplit_init(Dictionary afdict)
 	as->nparts = atoi(regparts->string[0]);
 	if (as->nparts < 0)
 	{
+		free_anysplit(afdict);
 		prt_error("Error: File %s: Value of %s must be a non-negative number",
 		          afdict->name, afdict_classname[AFDICT_REGPARTS]);
 		return false;
 	}
 	if (0 == as->nparts)
 	{
+		free_anysplit(afdict);
 		prt_error("Warning: File %s: Anysplit disabled (0: %s)\n",
 		          afdict->name, afdict_classname[AFDICT_REGPARTS]);
 		return true;
@@ -391,6 +396,7 @@ bool anysplit_init(Dictionary afdict)
 
 	if (2 != regalts->length)
 	{
+		free_anysplit(afdict);
 		prt_error("Error: File %s: Must have %s defined with 2 values",
 		          afdict->name, afdict_classname[AFDICT_REGALTS]);
 		return false;
@@ -399,6 +405,7 @@ bool anysplit_init(Dictionary afdict)
 	as->altsmax = atoi(regalts->string[1]);
 	if ((atoi(regalts->string[0]) <= 0) || (atoi(regalts->string[1]) <= 0))
 	{
+		free_anysplit(afdict);
 		prt_error("Error: File %s: Value of %s must be 2 positive numbers",
 		          afdict->name, afdict_classname[AFDICT_REGALTS]);
 		return false;
@@ -415,8 +422,9 @@ bool anysplit_init(Dictionary afdict)
  * - an error occurs (the behavior then is undefined).
  *   Such an error has not been observed yet.
  */
-bool anysplit(Sentence sent, const char *word)
+bool anysplit(Sentence sent, Gword *unsplit_word)
 {
+	const char * word = unsplit_word->subword;
 	Dictionary afdict = sent->dict->affix_table;
 	anysplit_params *as;
 	Afdict_class * stemsubscr;
@@ -437,7 +445,6 @@ bool anysplit(Sentence sent, const char *word)
 	bool use_sampling = true;
 	const char infix_mark = INFIX_MARK(afdict);
 
-
 	if (NULL == afdict) return false;
 	as = afdict->anysplit;
 
@@ -562,12 +569,10 @@ bool anysplit(Sentence sent, const char *word)
 		}
 
 		/* Here a leading INFIX_MARK is added to the suffixes if needed. */
-		add_alternative(sent,
-		   0,NULL, 1,(const char **)&prefix_string, num_suffixes,suffixes);
+		issue_word_alternative(sent, unsplit_word, "AS",
+		   0,NULL,  1,(const char **)&prefix_string, num_suffixes,suffixes);
 		free(suffixes);
 	}
 
 	return true;
 }
-
-#endif /* USE_ANYSPLIT */
