@@ -38,6 +38,7 @@
 #define MAX_STRIP 10
 #define SYNTHETIC_SENTENCE_MARK '>' /* A marking of a synthetic sentence. */
 #define D_SW 3                      /* debug level for word splits */
+#define D_UN 3                      /* debug level for units/punct */
 
 /* These are no longer in use, but are read from the 4.0.affix file */
 /* I've left these here, as an example of what to expect. */
@@ -335,7 +336,7 @@ static bool word_start_another_alternative(Dictionary dict,
 		    ((0 == strncmp((*n)->subword, altword0, strlen((*n)->subword))) &&
 			 !find_word_in_dict(dict, altword0))))
 		{
-			lgdebug(+2, "Preventing alt starts with %s due to existing %s\n",
+			lgdebug(+D_UN, "Preventing alt starts with %s due to existing %s\n",
 			        altword0, (*n)->subword);
 			return true;
 		}
@@ -965,6 +966,8 @@ error:
  * If unsplit_word is null, this function actually only checks whether
  * the alternative is valid as described above. This is used for checking
  * is a spell guess result if valid if the word itself is not in the dict.
+ * FIXME: If a word can split it doesn't follow it is a "real" dictionary
+ * word, as there can still be no links between some of its parts.
  *
  * Return true if the alternative is valid, else false.
  */
@@ -984,13 +987,19 @@ static bool add_alternative_with_subscr(Sentence sent,
 
 	if (0 == stemsubscr_count)
 	{
-		word_is_in_dict = true;
 		if (issue_alternatives)
 		{
+			word_is_in_dict = true;
 			issue_word_alternative(sent, unsplit_word, "AWS",
 			                       (prefix ? 1 : 0),&prefix, 1,&word,
 			                       (suffix ? 1 : 0),&suffix);
 		}
+		else
+		{
+			/* This is a compound-word spell check. Reject unknown words.
+			 * XXX: What if the word is capitalized? */
+			word_is_in_dict = boolean_dictionary_lookup(dict, word);
+		}
 	}
 	else
 	{
@@ -1033,11 +1042,6 @@ static bool add_alternative_with_subscr(Sentence sent,
  * It can also split contracted words (like he's).
  * Alternatives are generated if issue_alternatives=true.
  * Return value:
- * If issue_alternatives=true: true only if the word can morpheme-split.
- * If issue_alternatives=false: true only if the word can split.
- *
- * FIXME: If a word can split it doesn't follow it is a "real" dictionary word,
- * as there can still be no links between some of its parts.
  *
  * The prefix code is only lightly validated by actual use.
  *
@@ -1096,10 +1100,6 @@ static bool suffix_split(Sentence sent, Gword *unsplit_word, const char *w)
 				 * In case we try to split a contracted word, the first word
 				 * may match a regex. Hence find_word_in_dict() is used and
 				 * not boolean_dictionary_lookup().
-				 * However, if this is a check whether the word is a known one
-				 * (argument issue_alternatives=false), we shouldn't allow words
-				 * which are not in the dict file.
-				 *
 				 * Note: Not like a previous version, stems cannot match a regex
 				 * here, and stem capitalization need to be handled elsewhere. */
 				if ((is_contraction_word(w) &&
@@ -1129,8 +1129,10 @@ static bool suffix_split(Sentence sent, Gword *unsplit_word, const char *w)
 			for (j = 0; j < p_strippable; j++)
 			{
 				size_t prelen = strlen(prefix[j]);
-				/* The remaining w is too short for a possible match. */
-				if ((wend-w) - suflen < prelen) continue;
+				/* The remaining w is too short for a possible match.
+				 * NOTE: A zero length "stem" is not allowed here. In any
+				 * case, it cannot be handled (yet) by the rest of the code. */
+				if ((wend-w) - suflen <= prelen) continue;
 				if (strncmp(w, prefix[j], prelen) == 0)
 				{
 					size_t sz = MIN((wend-w) - suflen - prelen, MAX_WORD);
@@ -1327,7 +1329,7 @@ static bool mprefix_split(Sentence sent, Gword *unsplit_word, const char *word)
 				{
 					word_is_in_dict = true;
 					/* Add the prefix alone */
-					lgdebug(+3, "Whole-word prefix: %s\n", word);
+					lgdebug(+D_UN, "Whole-word prefix: %s\n", word);
 					if (split_check) return true;
 					altp = issue_word_alternative(sent, unsplit_word, "MPW",
 					          split_prefix_i+1,split_prefix, 0,NULL, 0,NULL);
@@ -1339,7 +1341,7 @@ static bool mprefix_split(Sentence sent, Gword *unsplit_word, const char *word)
 				if (find_word_in_dict(dict, newword))
 				{
 					word_is_in_dict = true;
-					lgdebug(+3, "Splitting off a prefix: %.*s-%s\n",
+					lgdebug(+D_UN, "Splitting off a prefix: %.*s-%s\n",
 					        wordlen-sz, word, newword);
 					if (split_check) return true;
 					altp = issue_word_alternative(sent, unsplit_word, "MPS",
@@ -1580,7 +1582,7 @@ static const char *strip_left(Sentence sent, const char * w,
 
 			if (strncmp(w, lpunc[i], sz) == 0)
 			{
-				lgdebug(2, "w='%s' found lpunc '%s'\n", w, lpunc[i]);
+				lgdebug(D_UN, "w='%s' found lpunc '%s'\n", w, lpunc[i]);
 				r_stripped[(*n_r_stripped)++] = lpunc[i];
 				w += sz;
 				break;
@@ -1717,7 +1719,7 @@ static bool strip_right(Sentence sent,
 
 			if (strncmp(temp_wend-len, t, len) == 0)
 			{
-				lgdebug(2, "%d: strip_right(%s): w='%s' rword '%s'\n",
+				lgdebug(D_UN, "%d: strip_right(%s): w='%s' rword '%s'\n",
 				        p, afdict_classname[classnum], temp_wend-len, t);
 				r_stripped[*n_r_stripped+nrs] = t;
 				nrs++;
@@ -1740,7 +1742,7 @@ static bool strip_right(Sentence sent,
 	 * since it is invoked with the last byte... */
 	if (rootdigit && (temp_wend > w) && !is_utf8_digit(temp_wend-1))
 	{
-		lgdebug(2, "%d: strip_right(%s): return FALSE; root='%s' (%c is not a digit)\n",
+		lgdebug(D_UN, "%d: strip_right(%s): return FALSE; root='%s' (%c is not a digit)\n",
 			 p, afdict_classname[classnum], word, temp_wend[-1]);
 		return false;
 	}
@@ -1753,7 +1755,7 @@ static bool strip_right(Sentence sent,
 		temp_wend += len;
 	}
 
-	lgdebug(2, "%d: strip_right(%s): return %s; n_r_stripped=%d+%d, wend='%s' temp_wend='%s'\n",
+	lgdebug(D_UN, "%d: strip_right(%s): return %s; n_r_stripped=%d+%d, wend='%s' temp_wend='%s'\n",
 p, afdict_classname[classnum],stripped?"TRUE":"FALSE",(int)*n_r_stripped,(int)nrs,*wend,temp_wend);
 
 	*n_r_stripped += nrs;
@@ -2154,7 +2156,12 @@ static void separate_word(Sentence sent, Gword *unsplit_word, Parse_Options opts
 	/* OK, now try to strip affixes. */
 	word_can_split = morpheme_split(sent, unsplit_word, word);
 
-	if (!word_is_known)
+	/* If the word is unknown, then try to guess its category by regexes.
+	 * A word that can split is considered known, unless it is a contraction,
+	 * in which case we need a regex for things like 1960's.
+	 * The first regex which matches (if any) is used.
+	 * An alternative consisting of the word has already been generated. */
+	if (!word_is_known && (!word_can_split || is_contraction_word(word)))
 	{
 		const char *regex_name = match_regex(dict->regex_root, word);
 		if ((NULL != regex_name) && boolean_dictionary_lookup(dict, regex_name))
@@ -2252,7 +2259,7 @@ static void separate_word(Sentence sent, Gword *unsplit_word, Parse_Options opts
 
 			if (!word_can_split && !word_is_known &&
 				 (!word_is_capitalizable || (lc_word_is_in_dict &&
-					(is_entity(dict, word) || is_common_entity(dict, downcase)))))
+					(is_common_entity(dict, downcase) || is_entity(dict, word)))))
 			{
 				/* Issue it (capitalized) too */
 				if ((NULL != unsplit_word->regex_name))
